@@ -301,6 +301,15 @@ sk_init_auth_widget (GladeXML *xml,
 		if (value && strlen (value))
 			gtk_entry_set_text (GTK_ENTRY (widget), value);
 	}
+
+	widget = glade_xml_get_widget (xml, "sk_remote_address_entry");
+	gtk_size_group_add_widget (group, widget);
+	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (changed_cb), user_data);
+	if (s_vpn) {
+		value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP);
+		if (value && strlen (value))
+			gtk_entry_set_text (GTK_ENTRY (widget), value);
+	}
 }
 
 static gboolean
@@ -415,6 +424,16 @@ auth_widget_check_validity (GladeXML *xml, const char *contype, GError **error)
 			             NM_OPENVPN_KEY_LOCAL_IP);
 			return FALSE;
 		}
+
+		widget = glade_xml_get_widget (xml, "sk_remote_address_entry");
+		str = gtk_entry_get_text (GTK_ENTRY (widget));
+		if (!str || !strlen (str)) {
+			g_set_error (error,
+			             OPENVPN_PLUGIN_UI_ERROR,
+			             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+			             NM_OPENVPN_KEY_REMOTE_IP);
+			return FALSE;
+		}
 	} else
 		g_assert_not_reached ();
 
@@ -487,6 +506,7 @@ auth_widget_update_connection (GladeXML *xml,
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	GtkWidget *widget;
+	const char *str;
 
 	if (!strcmp (contype, NM_OPENVPN_CONTYPE_TLS)) {
 		update_tls (xml, "tls", s_vpn);
@@ -497,7 +517,10 @@ auth_widget_update_connection (GladeXML *xml,
 		update_tls (xml, "pw_tls", s_vpn);
 		update_username (xml, "pw_tls", s_vpn);
 	} else if (!strcmp (contype, NM_OPENVPN_CONTYPE_STATIC_KEY)) {
+		/* Update static key */
 		update_from_filechooser (xml, NM_OPENVPN_KEY_STATIC_KEY, "sk", "key_chooser", s_vpn);
+
+		/* Update direction */
 		widget = glade_xml_get_widget (xml, "sk_direction_combo");
 		g_assert (widget);
 		model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
@@ -511,6 +534,20 @@ auth_widget_update_connection (GladeXML *xml,
 				g_free (tmp);
 			}
 		}
+
+		/* Update local address */
+		widget = glade_xml_get_widget (xml, "sk_local_address_entry");
+		g_assert (widget);
+		str = gtk_entry_get_text (GTK_ENTRY (widget));
+		if (str && strlen (str))
+			nm_setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP, str);
+
+		/* Update remote address */
+		widget = glade_xml_get_widget (xml, "sk_remote_address_entry");
+		g_assert (widget);
+		str = gtk_entry_get_text (GTK_ENTRY (widget));
+		if (str && strlen (str))
+			nm_setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP, str);
 	} else
 		g_assert_not_reached ();
 
@@ -884,7 +921,7 @@ populate_hmacauth_combo (GtkComboBox *box, const char *hmacauth)
 		NULL
 	};
 
-	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
+	store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
 	gtk_combo_box_set_model (box, GTK_TREE_MODEL (store));
 
 	/* Add default option which won't pass --auth to openvpn */
